
import java.util.Random;
import pulpcore.animation.Fixed;
import pulpcore.image.Colors;
import pulpcore.scene.Scene2D;
import pulpcore.sprite.FilledSprite;
import pulpcore.sprite.Label;
import pulpcore.Stage;

public class BubbleMark extends Scene2D {

    Fixed frameRate = new Fixed();
    Medio medium;
    int nBacteria = 1000;
    double coeficient = 1e-7;
    double reactionSpeed = 0.5;
    FilledSprite[][] matrizSprite;
    FilledSprite[] bacteriaSprite;
    double simulationTime;
    double simulationIncreased = 0.1;

    @Override
    public void load() {
        //Init medium
        medium = new Medio(5,5,nBacteria,coeficient,reactionSpeed);
        matrizSprite = new FilledSprite[Medio.M][Medio.N];
        bacteriaSprite = new FilledSprite[medium.getBacterias().size()];

        //Init simulation time
        simulationTime = 0;

        // Add background
        add(new FilledSprite(Colors.WHITE));

        //Paint
        paintMedio();
        paintBacteria();

        // Add fps display
        Label frameRateLabel = new Label("%.1f seconds", 5, 5);
        frameRateLabel.setFormatArg(frameRate);
        add(frameRateLabel);

        Stage.setFrameRate(1);
    }

    @Override
    public synchronized void update(int elapsedTime) {
        simulationTime += simulationIncreased;

        medium.siguienteIteracion(simulationIncreased);
        borrarMatriz();
        paintMedio();
        paintBacteria();

        // Update fps display
        double fps = Stage.getActualFrameRate();
        if (fps >= 0) {
            frameRate.set(simulationTime);
        }
    }

    public void borrarMatriz() {
        for (int i = 0; i < Medio.getM(); i++) {
            for (int j = 0; j < Medio.getN(); j++) {
                remove(matrizSprite[i][j]);
            }
        }
        int i = 0;
        for (IBacteria b : medium.getBacterias()) {
            remove(bacteriaSprite[i]);
            i++;
        }
    }

    public void paintMedio() {
        //Paint
        for (int i = 0; i < Medio.getM(); i++) {
            for (int j = 0; j < Medio.getN(); j++) {
                int intensidad = 0;
                if (2 * (int) medium.getMatriz()[i][j] <= 255) {
                    intensidad = 2 * (int) medium.getMatriz()[i][j];
                    matrizSprite[i][j] = new FilledSprite(Colors.rgba(255, 255 - intensidad, 255 - intensidad, 255));
                } else {
                    intensidad = 2 * (int) medium.getMatriz()[i][j] - 255;
                    if (255 - intensidad >= 0) {
                        matrizSprite[i][j] = new FilledSprite(Colors.rgba(255 - intensidad, 0, 0, 255));
                    } else {
                        matrizSprite[i][j] = new FilledSprite(Colors.rgba(0, 0, 0, 255));
                    }

                }
                matrizSprite[i][j].setLocation(100 + 31 * i, 30 + 31 * j);
                matrizSprite[i][j].setSize(30, 30);
                add(matrizSprite[i][j]);
            }
        }

    }

    public void paintBacteria() {
        Random rnd = new Random();
        int i = 0;
        for (IBacteria b : medium.getBacterias()) {
                bacteriaSprite[i] = new FilledSprite(Colors.rgba(121, 255, 79, 150));


                int x = rnd.nextInt(28);
                int y = rnd.nextInt(28);

                bacteriaSprite[i].setLocation(100 + x + 31 * b.getX(), 30 + y + 31 * b.getY());
                bacteriaSprite[i].setSize(4, 4);
                add(bacteriaSprite[i]);
                i++;
        }
    }

    // Methods called from JavaScript (invoke in the animation thread)
    public void setNumBacteria(final int numBacteria) {
        invokeLater(new Runnable() {

            public void run() {
                nBacteria = numBacteria;
                reload();
            }
        });
    }

    public void setCoeficient(final double newCoeficient) {
        invokeLater(new Runnable() {

            public void run() {
                coeficient = newCoeficient;
                reload();
            }
        });
    }

    public void setReactionSpeed(final double newReactionSpeed) {
        invokeLater(new Runnable() {

            public void run() {
                reactionSpeed = newReactionSpeed;
                reload();
            }
        });
    }

}
