

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Luis Eduardo Pavón Gómez and Luis Merino
 */
public class Medio implements IMedio {

    /* Tamaño de la matriz en celdas*/
    public static final int M = 11;
    public static final int N = 11;

    /*Tamño real del medio (metros)*/
    public static final double tamM = 0.01;
    public static final double tamN = 0.01;

    /* Coeficiente de difusion (en 2 dimensiones, metros/seg)*/
    public static double COEFICIENTE;

    /*Concentración (en 2D, mol/metros^2)*/
    private double[][] matriz;
    private int xInicial;
    private int yInicial;

    private List<IBacteria> bacterias;


    public Medio(int x, int y, int numBacterias, double Coeficiente, double reactionSpeed) {
        COEFICIENTE = Coeficiente;
        matriz = new double[M][N];
        bacterias = new ArrayList<IBacteria>();
      //inicializarMatriz(x, y);
        inicializarMatriz(matriz);
        setxInicial(x);
        setyInicial(y);
        crearBacterias(numBacterias, reactionSpeed);
        IBacteria b = new Bacteria(M,N,x,y,getMatriz()[x][y],tamN, reactionSpeed);
        b.setAttached(1);
        bacterias.add(b);
    }

    public static int getM() {
        return M;
    }

    public static int getN() {
        return N;
    }

    public static double getCOEFICIENTE() {
        return COEFICIENTE;
    }

    public int getxInicial() {
        return xInicial;
    }

    public void setxInicial(int xInicial) {
        this.xInicial = xInicial;
    }

    public int getyInicial() {
        return yInicial;
    }

    public void setyInicial(int yInicial) {
        this.yInicial = yInicial;
    }

    public double[][] getMatriz() {
        return matriz;
    }

    private void setMatriz(double[][] matriz) {
        this.matriz = matriz;
    }

    public List<IBacteria> getBacterias() {
        return bacterias;
    }

    public void setBacterias(List<IBacteria> bacterias) {
        this.bacterias = bacterias;
    }

    public void reiniciarSistema(int x, int y) {
        inicializarMatriz(x, y);
    }

    private void inicializarMatriz(int x, int y) {

        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                if (i != x || j != y) {
                    matriz[i][j] = 0;
                } else {
                    matriz[i][j] = 1;
                }
            }
        }
    }

    public void siguienteIteracion(double deltat) {
        difundir(deltat);
        aumentarConcentracion(deltat);
        imprimirMedio();
        iterarBacterias(deltat);
        imprimirBacterias();
    }

    private void difundir(double dt) {
        double[][] aux = new double[M][N];
        inicializarMatriz(aux);

        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                if (matriz[i][j] != 0) {
                    Posicion p = calcularPosiciones(i, j);
                    difundirCelda(i, j, p, aux, dt);
                }
            }
        }

        matriz = aux;
    }

    private Posicion calcularPosiciones(int x, int y) {
        Posicion p = new Posicion();
        double actual = matriz[x][y];

        int k = 0;
        for (int i = -1; i <= 1; i++) {
            for (int j = -1; j <= 1; j++) {
                if (esValido(x + i, y + j)) {
                    if (actual > matriz[x + i][y + j]) {
                        boolean[] aux = p.getPosiciones();
                        aux[k] = true;
                        p.setPosiciones(aux);
                        p.setPosicionesCorrectas(p.getPosicionesCorrectas() + 1);
                    }
                } else {
                    p.getPosiciones()[k] = false;
                }
                k++;
            }
        }

        return p;
    }

    private boolean esValido(int x, int y) {
        boolean valido = false;

        if (x >= 0 && x < M && y >= 0 && y < N) {
            valido = true;
        }

        return valido;
    }

    private void difundirCelda(int x, int y, Posicion p, double[][] aux, double dt) {
        int k = 0;
        double valor = 0;
        double D=COEFICIENTE*N*M/(tamN*tamM);
        for (int i = -1; i <= 1; i++) {
            for (int j = -1; j <= 1; j++) {
                if (i != 0 || j != 0) {
                    if (p.getPosiciones()[k]) {
                        double valorAdifundir =dt*(matriz[x][y] - matriz[x+i][y+j]) * D;//COEFICIENTE;
                        aux[x + i][y + j] += valorAdifundir;
                        valor += valorAdifundir;
                    }
                }
                k++;
            }
        }
        aux[x][y] += matriz[x][y] - valor;
    }

    /*private void aumentarConcentracion(double dt) {
        matriz[getxInicial()][getyInicial()] += dt*getAUMENTO();
    }*/

     private void aumentarConcentracion(double dt) {
        for(IBacteria b : getBacterias()){
           matriz[b.getX()][b.getY()]+=b.getIncrementoCA(dt);
        }
    }

    private void crearBacterias(int numBacterias, double reactionSpeed) {
        Random rnd = new Random();
        for(int i=0;i<numBacterias;i++){
            int x = rnd.nextInt(3);
            int y = rnd.nextInt(3);
            IBacteria b = new Bacteria(M,N,x,y,getMatriz()[x][y],tamN, reactionSpeed);
            bacterias.add(b);
        }
    }

    public void iterarBacterias(double dt){
        for(IBacteria b : getBacterias()){
           b.calcularProbabilidadCarrera(getMatriz()[b.getX()+b.getMovimientoX()][b.getY()+b.getMovimientoY()]);
           b.iterar(dt);
           b.comprobarConcentracion(getMatriz()[b.getX()][b.getY()]);
           b.setState(xInicial-1,xInicial,yInicial-1,yInicial);
        }
    }

    public void imprimirMedio() {
        DecimalFormat df = new DecimalFormat("0.000");
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                System.out.print(df.format(matriz[i][j]) + "\t");
            }
            System.out.println("\n");
        }
    }

    public void imprimirBacterias() {
        int[][] mBacterias = new int[M][N];
        inicializarMatriz(mBacterias);
        for(IBacteria b : getBacterias()){
           mBacterias[b.getX()][b.getY()] += 1;
        }
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                System.out.print(mBacterias[i][j] + "\t");
            }
            System.out.println("\n");
        }
    }

    public String stringBacterias() {
        int[][] mBacterias = new int[M][N];
        String s = "";

        inicializarMatriz(mBacterias);
        for(IBacteria b : getBacterias()){
           mBacterias[b.getX()][b.getY()] += 1;
        }
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                s += mBacterias[i][j] + "\t";
            }
            s += "\n";
        }

        return s;
    }
    private void inicializarMatriz(double[][] aux) {
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                aux[i][j] = 0;
            }
        }
    }

    private void inicializarMatriz(int[][] aux) {
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                aux[i][j] = 0;
            }
        }
    }

    @Override
    public String toString() {
        String s = "";
        DecimalFormat df = new DecimalFormat("0.00");

        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) {
                s += df.format(matriz[i][j]) + "\t";
            }
            s += "\n";
        }

        return s;
    }




}
